import sys

import atheris
from helpers import EnhancedDataProvider

with atheris.instrument_imports():
    from python_multipart.multipart import parse_options_header


def TestOneInput(data: bytes) -> None:
    fdp = EnhancedDataProvider(data)
    try:
        parse_options_header(fdp.ConsumeRandomBytes())
    except AssertionError:
        return
    except TypeError:
        return


def main():
    atheris.Setup(sys.argv, TestOneInput)
    atheris.Fuzz()


if __name__ == "__main__":
    main()



import io
import sys

import atheris
from helpers import EnhancedDataProvider

with atheris.instrument_imports():
    from python_multipart.decoders import Base64Decoder, DecodeError, QuotedPrintableDecoder


def fuzz_base64_decoder(fdp: EnhancedDataProvider) -> None:
    decoder = Base64Decoder(io.BytesIO())
    decoder.write(fdp.ConsumeRandomBytes())
    decoder.finalize()


def fuzz_quoted_decoder(fdp: EnhancedDataProvider) -> None:
    decoder = QuotedPrintableDecoder(io.BytesIO())
    decoder.write(fdp.ConsumeRandomBytes())
    decoder.finalize()


def TestOneInput(data: bytes) -> None:
    fdp = EnhancedDataProvider(data)
    targets = [fuzz_base64_decoder, fuzz_quoted_decoder]
    target = fdp.PickValueInList(targets)

    try:
        target(fdp)
    except DecodeError:
        return


def main():
    atheris.Setup(sys.argv, TestOneInput)
    atheris.Fuzz()


if __name__ == "__main__":
    main()



import io
import sys
from unittest.mock import Mock

import atheris
from helpers import EnhancedDataProvider

with atheris.instrument_imports():
    from python_multipart.exceptions import FormParserError
    from python_multipart.multipart import parse_form

on_field = Mock()
on_file = Mock()


def parse_octet_stream(fdp: EnhancedDataProvider) -> None:
    header = {"Content-Type": "application/octet-stream"}
    parse_form(header, io.BytesIO(fdp.ConsumeRandomBytes()), on_field, on_file)


def parse_url_encoded(fdp: EnhancedDataProvider) -> None:
    header = {"Content-Type": "application/x-url-encoded"}
    parse_form(header, io.BytesIO(fdp.ConsumeRandomBytes()), on_field, on_file)


def parse_form_urlencoded(fdp: EnhancedDataProvider) -> None:
    header = {"Content-Type": "application/x-www-form-urlencoded"}
    parse_form(header, io.BytesIO(fdp.ConsumeRandomBytes()), on_field, on_file)


def parse_multipart_form_data(fdp: EnhancedDataProvider) -> None:
    boundary = "boundary"
    header = {"Content-Type": f"multipart/form-data; boundary={boundary}"}
    body = (
        f"--{boundary}\r\n"
        f"Content-Type: multipart/form-data; boundary={boundary}\r\n\r\n"
        f"{fdp.ConsumeRandomString()}\r\n"
        f"--{boundary}--\r\n"
    )
    parse_form(header, io.BytesIO(body.encode("latin1", errors="ignore")), on_field, on_file)


def TestOneInput(data: bytes) -> None:
    fdp = EnhancedDataProvider(data)
    targets = [parse_octet_stream, parse_url_encoded, parse_form_urlencoded, parse_multipart_form_data]
    target = fdp.PickValueInList(targets)

    try:
        target(fdp)
    except FormParserError:
        return


def main():
    atheris.Setup(sys.argv, TestOneInput)
    atheris.Fuzz()


if __name__ == "__main__":
    main()



import atheris


class EnhancedDataProvider(atheris.FuzzedDataProvider):
    def ConsumeRandomBytes(self) -> bytes:
        return self.ConsumeBytes(self.ConsumeIntInRange(0, self.remaining_bytes()))

    def ConsumeRandomString(self) -> str:
        return self.ConsumeUnicodeNoSurrogates(self.ConsumeIntInRange(0, self.remaining_bytes()))



import inspect

import nox

nox.needs_version = ">=2024.4.15"
nox.options.default_venv_backend = "uv|virtualenv"


@nox.session
@nox.parametrize("editable", [True, False])
def rename(session: nox.Session, editable: bool) -> None:
    session.install("-e." if editable else ".")
    # Ensure warning is not visible by default
    assert "import python_multipart" not in session.run("python", "-c", "import multipart", silent=True)

    assert "import python_multipart" in session.run("python", "-Wdefault", "-c", "import multipart", silent=True)
    assert "import python_multipart" in session.run(
        "python", "-Wdefault", "-c", "import multipart.exceptions", silent=True
    )
    assert "import python_multipart" in session.run(
        "python", "-Wdefault", "-c", "from multipart import exceptions", silent=True
    )
    assert "import python_multipart" in session.run(
        "python", "-Wdefault", "-c", "from multipart.exceptions import FormParserError", silent=True
    )

    session.install("multipart")
    assert "import python_multipart" not in session.run(
        "python", "-Wdefault", "-c", "import multipart; multipart.parse_form_data", silent=True
    )
    assert "import python_multipart" not in session.run(
        "python", "-Wdefault", "-c", "import python_multipart; python_multipart.parse_form", silent=True
    )


@nox.session
def rename_inline(session: nox.Session) -> None:
    session.install("pip")
    res = session.run(
        "python",
        "-Wdefault",
        "-c",
        inspect.cleandoc("""
        import subprocess

        subprocess.run(["pip", "install", "."])

        import multipart
    """),
        silent=True,
    )
    assert "Please use `import python_multipart` instead." in res
